
#
#   Install dependencies
# $ conan install .
#
#   Conan build dependencies
# $ conan install . --build
#
#   Conan build
# $ conan build .
#
#   Build from cmake
# $ cmake --build ./bld -j 8
#
#   install / build / package
# $ ./clean.sh && conan install . && conan build . && conan package .
#
#   Install / Uninstall
# $ sudo cmake --install ./bld && sudo ./run.sh uninstall
#

import datetime
import platform
from conans import ConanFile, tools, CMake

# Read from config file
def readConfig(fname):
    cfg = {}
    with open(fname) as f:
        lines = f.readlines()
        for line in lines:
            parts = line.strip().replace("\t", " ").split(" ")
            k = parts.pop(0).strip()
            cfg[k] = " ".join(parts).strip()
    return cfg


class Library_pymembus(ConanFile):

    # Read config file
    cfg = readConfig('PROJECT.txt')
    for k in ['name', 'version', 'description', 'url', 'license', 'company', 'author']:
        if k not in cfg:
            raise Exception("Variable '%s' not specified in 'PROJECT.txt'" % k)
        locals()[k] = cfg[k]

    # Create a build number
    buildno = datetime.datetime.now().strftime("%y.%m.%d.%H%M").replace(".0", ".")

    # requires = ()
    settings = "os", "compiler", "arch", "build_type"
    options = {}
    default_options = ""
    exports = "*"
    generators = "cmake"
    build_policy = "missing"
    build_output = "./bld"

    if platform.system() == "Windows":
        install_path = "C:/Program Files/%s" % cfg['name']
    else:
        install_path = "/usr/local"

    def conan_info(self):
        self.info.settings.clear()

    def configure(self):
        if self.settings.compiler == "gcc":
            self.settings.compiler.libcxx = "libstdc++11"

    def build(self):

        print("COMPILER: %s %s %s" 
                % (self.settings.compiler, self.settings.compiler.version, self.settings.compiler.libcxx))

        # https://docs.conan.io/en/latest/reference/build_helpers/cmake.html#definitions
        cmake = CMake(self, parallel=True)
        # cmake.verbose = True
        cmake.definitions["APPBUILD"] = self.buildno
        cmake.definitions["CMAKE_INSTALL_PREFIX"] = self.install_path
        cmake.configure(build_dir=self.build_output)

        print("cmake.command_line : %s" % cmake.command_line)
        print("cmake.build_config : %s" % cmake.build_config)

        cmake.build()
        # cmake.install()

    def package(self):
        self.run('cpack -B ./pck -G DEB -C Release --config %s/CPackConfig.cmake' % self.build_output)

    def package_info(self):
        print(self.package_folder)




#!/usr/bin/env python3

import os
import sys
import time
import argparse
import inspect
import traceback

# csp = os.path.dirname(os.path.realpath(__file__))
# print(csp)
# sys.path.append(csp)

# C:/Users/bob/AppData/Local/Temp/pip-req-build-1ldcogu6/_skbuild/win-amd64-3.9/cmake-build/lib/Release/test.py
# C:\Users\bob\AppData\Local\Temp\pip-req-build-1ldcogu6\_skbuild\win-amd64-3.9\cmake-build\lib\Release\test.py
isNumpy = False
try:
    import numpy
    isNumpy = True
except Exception as e:
    isNumpy = False

def exitWithError(s):

    text = ""

    if isinstance(s, Exception):

        try:
            tb = traceback.TracebackException.from_exception(s)
            dep = len(tb.stack)
            filename = os.path.basename(str(tb.stack[dep-1][0]))
            lineno = str(tb.stack[dep-1][1])
            reason = str(tb._str)
            if not reason:
                reason = str(tb.exc_type)
            stack = ""
            for st in tb.stack:
                stack += "  %s(%s): %s() : %s\n" % (st[0], st[1], st[2], st[3])
            text = "%s(%s): [\033[91mERROR\033[0m] %s\n%s" % (filename, lineno, reason, stack)

        except Exception as e:
            st = inspect.stack()
            dep = len(st)
            fi = inspect.getframeinfo(st[dep-1][0])
            reason = traceback.format_exc()
            text += "\n%s:%s(%s): [\033[91mERROR\033[0m] %s" % (os.path.basename(fi.filename), fi.function, fi.lineno, reason)

    if not text:
        st = inspect.stack()
        fi = inspect.getframeinfo(st[1][0])
        text += "%s:%s(%s): [\033[91mERROR\033[0m] %s" % (os.path.basename(fi.filename), fi.function, fi.lineno, s)

    print(text)
    exit(-1)


def runTests(_p):

    print("\n------------------------- Starting tests -------------------------")
    print("PARAMETERS  : %s" % _p)
    print("PYTHON      : %s" % sys.version.replace("\n", ""))
    print("WORKING DIR : %s" % os.getcwd())
    print("------------------------------------------------------------------")

    # Assert macros
    assert True
    assert not False

    # from pymembus import pymembus
    import pymembus
    if hasattr(pymembus, "pymembus"):
        pymembus = pymembus.pymembus

    # Module attributes
    print(dir(pymembus))

    # for k in dir(pymembus):
    #     print("%s = %s" % (k, getattr(pymembus, k)))

    print("[%s] %s" % (pymembus.__name__, pymembus.__file__))

    # Show version number
    print("pymembus version : %s [%s]" % (pymembus.__version__, pymembus.__build__))

    #----------------------------------------------------------------
    # Map shared memory
    sm = pymembus.memmap()
    assert sm.open("/myshare", 1024, True)

    # Write string
    msg = str(time.time())
    assert 0 < sm.write(msg)

    # Read the string
    assert sm.read(len(msg)) == msg

    if isNumpy:
        mem = numpy.array(sm, copy = False)
        print("MEM: %s" % str(mem))

    sm.close()


    #----------------------------------------------------------------
    # Shared message buffer
    tx = pymembus.memmsg()
    assert tx.open("/mymsg", 1024, True, True)
    assert not tx.existing()

    rx = pymembus.memmsg()
    assert rx.open("/mymsg", 1024, False, False)
    assert rx.existing()

    for i in range(0, 10):
        # print("WRITE: %s" % i)
        assert tx.write("Message %s" % i)

    i = 0
    max = 20
    while 10 > i and 0 < max:
        max -= 1
        msg = rx.read(0)
        if 0 < len(msg):
            # print("READ : %s" % msg)
            i += 1

    print("i = %s" % i)

    assert 10 <= i

    tx.close()
    rx.close()

    #----------------------------------------------------------------
    # Huge write test
    for b in range(0, 128):

        assert tx.open("/mymsg", 64 + b, True, True)
        assert not tx.existing()
        assert rx.open("/mymsg", 64 + b, False, False)
        assert rx.existing()

        for i in range(0, 1000):
            msg = "Message %s-%s" % (b, i)
            assert tx.write(msg)
            rmsg = rx.read(0)
            assert rmsg == msg

        tx.close()
        rx.close()

    for b in range(128, 0):

        assert tx.open("/mymsg", 64 + b, True, True)
        assert not tx.existing()
        assert rx.open("/mymsg", 64 + b, False, False)
        assert rx.existing()

        for i in range(0, 1000):
            msg = "Message %s-%s" % (b, i)
            assert tx.write(msg)
            rmsg = rx.read(0)
            assert rmsg == msg

        tx.close()
        rx.close()

    #----------------------------------------------------------------
    # Shared image
    imgs = pymembus.memvid()

    # Make sure getBuf() on invalid object throws
    asserted = False
    try:
        img = imgs[0]
    except Exception as e:
        asserted = True
    assert asserted

    # Open a share
    w = 320
    h = 240
    bpp = 24
    fps = 30
    bufs = 10
    assert imgs.open("/myvideo", True, w, h, bpp, fps, bufs)

    # Check values
    assert imgs.getWidth() == w
    assert imgs.getHeight() == h
    assert imgs.getBpp() == bpp
    assert imgs.getFps() == fps
    assert imgs.getBufs() == bufs
    assert imgs.getPtr(0) == 0
    assert imgs.next(1) == 1
    assert imgs.next(-1) == 0
    assert imgs.getPtrErr(3, 1) == 2
    assert len(imgs) == bufs

    # Check second share
    imgs2 = pymembus.memvid()
    assert imgs2.open_existing("/myvideo")
    assert imgs2.getWidth() == w
    assert imgs2.getHeight() == h
    assert imgs2.getBpp() == bpp
    assert imgs2.getFps() == fps
    assert imgs2.getBufs() == bufs
    imgs2.close()

    if isNumpy:

        # Write into the image buffer
        img = numpy.array(imgs[0], copy = False)

        try:
            import cv2
            cv2.rectangle(img, (10, 10), (60, 60), (50,255,50), -1)
            cv2.rectangle(img, (50, 50), (110, 110), (255,50,50), -1)
            cv2.rectangle(img, (100, 100), (150, 150), (50,50,255), -1)

        except Exception as e:
            print(e)


    #----------------------------------------------------------------
    # Shared image
    auds = pymembus.memaud()

    ch = 2
    bps = 16
    abr = 48000
    fps = 50
    bufs = 10
    assert auds.open("/myaudio", True, ch, bps, abr, fps, bufs)
    assert auds.getChannels() == ch
    assert auds.getBps() == bps
    assert auds.getBitRate() == abr
    assert auds.getFps() == fps
    assert auds.getBufs() == bufs
    assert len(auds) == bufs
    print("%s -- %s" % (auds.getBufSize(), (abr / fps)))
    assert auds.getBufSize() == (abr / fps)

    auds2 = pymembus.memaud()
    assert auds2.open_existing("/myaudio")
    assert auds2.getChannels() == ch
    assert auds2.getBps() == bps
    assert auds2.getBitRate() == abr
    assert auds2.getFps() == fps
    assert auds2.getBufs() == bufs

    if isNumpy:

        aud = numpy.array(auds[0], copy = False)
        aud2 = numpy.array(auds2[0], copy = False)

        # Create numpy audio buffer
        msamples = int(abr / fps)
        nsamples = msamples * ch
        src = numpy.zeros((1, nsamples), dtype='int16')
        src.fill(0)

        dch, dsz = aud.shape
        sch, ssz = src.shape

        # Ensure channel sizes match
        print("%s : %s --- %s : %s" % (dch, dsz, sch, ssz))
        assert dch == sch
        assert dsz == ssz

        # Check share
        aud[0][3] = 1234
        assert aud[0][3] == aud2[0][3]

        # Check the fill function
        auds.fill(0, 0)
        assert aud[0][3] == 0

        # Copy data
        aud = src

    auds.close()
    auds2.close()

    print("--- SUCCESS --- ")


def main():

    #---------------------------------------------------------------
    # Parse command line arguments
    ap = argparse.ArgumentParser(description='Test')
    ap.add_argument('--output', '-o', default='', type=str, help='Output file or directory')
    _p = vars(ap.parse_args())

    try:
        runTests(_p)

    except Exception as e:
        exitWithError(e)

if __name__ == '__main__':
    main()




#!/usr/bin/python3

# https://scikit-build.readthedocs.io/en/latest/usage.html

from __future__ import print_function

# from setuptools import setup
from skbuild import setup
import platform

def readConfig(fname):
    cfg = {}
    with open(fname) as f:
        lines = f.readlines()
        for line in lines:
            parts = line.strip().replace("\t", " ").split(" ")
            k = parts.pop(0).strip()
            cfg[k] = " ".join(parts).strip()
    return cfg

cfg = readConfig('PROJECT.txt')

setup(
    name=cfg['name'],
    version=cfg['version'],
    description=cfg['description'],
    url=cfg['url'],
    author=cfg['author'],
    author_email=cfg['email'],
    license=cfg['license'],
    # cmake_args=['-DPYVER==%s' % platform.python_version()],
)



#!/usr/bin/env python3

from __future__ import print_function

import argparse
from pymembus import pymembus

def readTest(_p):

    rx = pymembus.memmsg()

    if not rx.open("/mymsg", 1024, False, True):
        print("Failed to open share for reading: %s" % _p['share'])
        return -1

    if rx.existing():
        print("Share already exists: %s" % _p['share'])

    while True:

        msg = rx.read(5000)

        if msg:
            print("\n%s" % msg, flush=True)
        else:
            print(".", end='', flush=True)

    pass

def writeTest(_p):

    tx = pymembus.memmsg()

    if not tx.open("/mymsg", 1024, True, False):
        print("Failed to open share for writing: %s" % _p['share'])
        return -1

    if not tx.write(_p['message']):
        print("Failed to write message to share: %s" % _p['share'])
        return -1

    return 0


def main():

    #---------------------------------------------------------------
    # Parse command line arguments
    ap = argparse.ArgumentParser(description='Test')
    ap.add_argument('--read', '-r', action="store_true", help='Read test')
    ap.add_argument('--write', '-w', action="store_true", help='Write test')
    ap.add_argument('--share', '-s', default='/myshare', type=str, help='Share name')
    ap.add_argument('--message', '-m', default='This is a message.', type=str, help='Message to send')

    _p = vars(ap.parse_args())
    print("PARAMS: %s" %_p)

    if _p['read']:
        readTest(_p)

    elif _p['write']:
        writeTest(_p)

    else:
        print("You must specify --read or --write")


if __name__ == '__main__':
    main()




#!/usr/bin/env python3

import os
import sys
import time
import argparse
import inspect
import traceback

def exitWithError(s):

    text = ""

    if isinstance(s, Exception):

        try:
            tb = traceback.TracebackException.from_exception(s)
            dep = len(tb.stack)
            filename = os.path.basename(str(tb.stack[dep-1][0]))
            lineno = str(tb.stack[dep-1][1])
            reason = str(tb._str)
            if not reason:
                reason = str(tb.exc_type)
            stack = ""
            for st in tb.stack:
                stack += "  %s(%s): %s() : %s\n" % (st[0], st[1], st[2], st[3])
            text = "%s(%s): [\033[91mERROR\033[0m] %s\n%s" % (filename, lineno, reason, stack)

        except Exception as e:
            st = inspect.stack()
            dep = len(st)
            fi = inspect.getframeinfo(st[dep-1][0])
            reason = traceback.format_exc()
            text += "\n%s:%s(%s): [\033[91mERROR\033[0m] %s" % (os.path.basename(fi.filename), fi.function, fi.lineno, reason)

    if not text:
        st = inspect.stack()
        fi = inspect.getframeinfo(st[1][0])
        text += "%s:%s(%s): [\033[91mERROR\033[0m] %s" % (os.path.basename(fi.filename), fi.function, fi.lineno, s)

    print(text)
    exit(-1)


def runTests(_p):

    print("\n------------------------- Starting tests -------------------------")
    print("PARAMETERS  : %s" % _p)
    print("PYTHON      : %s" % sys.version.replace("\n", ""))
    print("WORKING DIR : %s" % os.getcwd())
    print("------------------------------------------------------------------")

    # Assert macros
    assert True
    assert not False

    # from pymembus import pymembus
    import pymembus
    if hasattr(pymembus, "pymembus"):
        pymembus = pymembus.pymembus

    # Module attributes
    print(dir(pymembus))

    # for k in dir(pymembus):
    #     print("%s = %s" % (k, getattr(pymembus, k)))

    print("[%s] %s" % (pymembus.__name__, pymembus.__file__))

    # Show version number
    print("pymembus version : %s [%s]" % (pymembus.__version__, pymembus.__build__))


    pt = pymembus.pytest()

    assert pt.add(2, 3) == 5

    print("--- SUCCESS --- ")


def main():

    #---------------------------------------------------------------
    # Parse command line arguments
    ap = argparse.ArgumentParser(description='Mosaic Creator')
    ap.add_argument('--output', '-o', default='', type=str, help='Output file or directory')
    _p = vars(ap.parse_args())

    try:
        runTests(_p)

    except Exception as e:
        exitWithError(e)

if __name__ == '__main__':
    main()



#!/usr/bin/python3

# https://scikit-build.readthedocs.io/en/latest/usage.html

# pip install . && ./test/test.py
# ./clean.sh && ./setup.py bdist_wheel
# ./clean.sh && ./setup.py sdist
# python3 -m twine upload -r testpypi dist/*
# python3 -m twine upload dist/*

from __future__ import print_function

import os
# from setuptools import setup
from skbuild import setup

def readConfig(fname):
    cfg = {}
    with open(fname) as f:
        lines = f.readlines()
        for line in lines:
            parts = line.strip().replace("\t", " ").split(" ")
            k = parts.pop(0).strip()
            if '#' != k[0]:
                cfg[k] = " ".join(parts).strip()
    return cfg

# Read in the config
here = os.path.abspath(os.path.dirname(__file__))
cfg = readConfig(os.path.join(here, 'PROJECT.txt'))

# Read in the README
with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()
if not long_description:
    long_description = cfg['description']

setup(
    name=cfg['name'],
    version=cfg['version'],
    description=cfg['description'],
    url=cfg['url'],
    author=cfg['author'],
    author_email=cfg['email'],
    license=cfg['license'],
    include_package_data = True,
    long_description=long_description,
    long_description_content_type='text/markdown'
    # packages=[],
    # install_requires=[],
    # cmake_args=['-DPYVER==%s' % platform.python_version()],
)
